pub(crate) struct Edge {
connections: SmallVec<[Connection; 1]>,
}Expand description
An edge in the graph
This connects one or more pair of ports between two
nodes, each connection represented by a Connection.
WebAudio allows for multiple connections to/from the same port
however it does not allow for duplicate connections between pairs
of ports
Fields
connections: SmallVec<[Connection; 1]>Implementations
Sourceimpl Edge
impl Edge
Sourcefn has_between(
&self,
output_idx: PortIndex<OutputPort>,
input_idx: PortIndex<InputPort>,
) -> bool
fn has_between( &self, output_idx: PortIndex<OutputPort>, input_idx: PortIndex<InputPort>, ) -> bool
Find if there are connections between two given ports, return the index
fn remove_by_output(&mut self, output_idx: PortIndex<OutputPort>)
fn remove_by_input(&mut self, input_idx: PortIndex<InputPort>)
fn remove_by_pair( &mut self, output_idx: PortIndex<OutputPort>, input_idx: PortIndex<InputPort>, )